use 5.008;
use ExtUtils::MakeMaker;
use English;

#< get the perl path, copied from man perlvar
use Config;
my $secure_perl_path = $Config{perlpath};
if ($^O ne 'VMS') {
    $secure_perl_path .= $Config{_exe} unless
	$secure_perl_path =~ m/$Config{_exe}$/i;
}
#>

my %prereq = (BerkeleyDB => 0,
	      MLDBM => 0);
my $AQUAINT_counts = 'lib/unigram-doc-counts-AQUAINT.txt';
my $dbmfile = '$(INST_LIBDIR)/unigram-idfs-AQUAINT.dbm';
use lib 'lib';
use dbm;

# See lib/ExtUtils/MakeMaker.pm for details of how to influence
# the contents of the Makefile that is written.
WriteMakefile(
	      NAME => 'Nuggeteer',
#	      VERSION_FROM => 'src/nuggeteer.pl', # finds $VERSION
	      VERSION => "0.8.1",
	      PREREQ_PM => {%prereq, 
			"porter"}, # e.g., Module::Name => 1.1
	      PM => { 'lib/Idf.pm' => '$(INST_LIBDIR)/Idf.pm',
		      'lib/porter.pm' => '$(INST_LIBDIR)/porter.pm',
		      'lib/dbm.pm' => '$(INST_LIBDIR)/dbm.pm',
		      'nuggeteer.pl' => '$(INST_SCRIPT)/nuggeteer.pl',
		      'genstats.pl' => '$(INST_SCRIPT)/genstats.pl',
		      'extractpernid.pl' => '$(INST_SCRIPT)/extractpernid.pl',
		      'cdft' => '$(INST_BIN)/cdft'
		  },
	      INC => 'lib',
	      clean => {FILES => 'genstats.pl nuggeteer.pl lib/makedbm.pl t/mock-thresholds.pl t/mock-judging.pl t/mock-crash.pl t/test-bigrams-0.15-c.out t/test-exact.out t/test-MITCSAIL.c-unknowns cdft extractpernid.pl t/test-modified-MITCSAIL.c-unknowns tiny-set/output.extt:t-mock-thresholds.pl tiny-set/output.t0.1 tiny-set/unknowns tiny-set/judgement-mock.output.arj tiny-set/output.t1 tiny-set/output.art tiny-set/output.err tiny-set/judgement-mock.output.t0.1 tiny-set/judgement-mock.output.exts:t-mock-judging.pl tiny-set/output tiny-set/judgement-mock.error tiny-set/judgement-mock.output.exts:t-mock-crash.pl tiny-set/judgement-mock.output'},
	      depend => { 'lib/Idf.pm' => $dbmfile,
			  'testing-prereqs' => 't/mock-thresholds.pl t/mock-judging.pl t/mock-crash.pl',
		         },

    ($] >= 5.005 ?     ## Add these new keywords supported since 5.005
      (ABSTRACT_FROM  => 'src/nuggeteer.pl', # retrieve abstract from module
       AUTHOR         => 'Gregory Adam Marton <gremio@mit.edu>') : ()),
);

sub MY::postamble {
    return <<"MAKE_FRAG";
$dbmfile: $AQUAINT_counts lib/makedbm.pl
	\@mkdir -p \$(INST_LIBDIR)
	\@echo "Building the idf hash may take 5 to 10 minutes:"
	perl -I lib lib/makedbm.pl --eval 'log(1033461/shift)' $AQUAINT_counts $dbmfile

nuggeteer.pl: src/nuggeteer.pl  testing-prereqs
	\@echo "src/nuggeteer.pl --> nuggeteer.pl"
	\@perl -p -e 's|^\\\#! \\S* perl \\s -w \\s* \$ |\\\#!$secure_perl_path -w\\n|x;' src/nuggeteer.pl > nuggeteer.pl
	\@chmod +x nuggeteer.pl

genstats.pl: src/genstats.pl cdft
	\@echo "src/genstats.pl --> genstats.pl"
	\@perl -p -e 's|^\\\#! \\S* perl \\s -w \\s* \$ |\\\#!$secure_perl_path -w\\n|x;' src/genstats.pl > genstats.pl
	\@chmod +x genstats.pl

lib/makedbm.pl: src/makedbm.pl
	\@echo "src/makedbm.pl --> lib/makedbm.pl"
	\@perl -p -e 's|^\\\#! \\S* perl \\s -w \\s* \$ |\\\#!$secure_perl_path -w\\n|x;' src/makedbm.pl > lib/makedbm.pl
	\@chmod +x lib/makedbm.pl

t/mock-thresholds.pl: src/mock-thresholds.pl
	\@echo "src/mock-thresholds.pl --> t/mock-thresholds.pl"
	\@perl -p -e 's|^\\\#! \\S* perl \\s -w \\s* \$ |\\\#!$secure_perl_path -w\\n|x;' src/mock-thresholds.pl > t/mock-thresholds.pl
	\@chmod +x t/mock-thresholds.pl

t/mock-judging.pl: src/mock-judging.pl
	\@echo "src/mock-judging.pl --> t/mock-judging.pl"
	\@perl -p -e 's|^\\\#! \\S* perl \\s -w \\s* \$ |\\\#!$secure_perl_path -w\\n|x;' src/mock-judging.pl > t/mock-judging.pl
	\@chmod +x t/mock-judging.pl

t/mock-crash.pl: src/mock-crash.pl
	\@echo "src/mock-crash.pl --> t/mock-crash.pl"
	\@perl -p -e 's|^\\\#! \\S* perl \\s -w \\s* \$ |\\\#!$secure_perl_path -w\\n|x;' src/mock-crash.pl > t/mock-crash.pl
	\@chmod +x t/mock-crash.pl

cdft: src/cdft.c
	gcc -lm -O -o cdft src/cdft.c

extractpernid.pl: src/extractpernid.pl
	\@echo "src/extractpernid.pl --> lib/extractpernid.pl"
	\@perl -p -e 's|^\\\#! \\S* perl \\s -w \\s* \$ |\\\#!$secure_perl_path -w\\n|x;' src/extractpernid.pl > extractpernid.pl
	\@chmod +x extractpernid.pl


MAKE_FRAG
    ;
}
