#!/usr/local/bin/perl -w 
package Idf;

#< pod
use constant VERSION => q$Id: Idf.pm,v 1.2 2006/01/18 18:39:25 gremio Exp $.$/;
use constant USAGE => <<"EOT"
usage: $0 [options] words
EOT
    ;
=pod

=head1 SYNOPSYS

From within another perl program: 

  use Idf qw(is_stopword words nonstopwords contentwords 
             lowfirst unique fmeas trim regexof); #idf and idfs by default
  print idf("word");
  print idfs(contentwords("the content words of this sentence"))

From the command line: 

  Idf.pm words
  Idf.pm --nonstopwords words
  Idf.pm --contentwords words
  Idf.pm --sanity f  prints those words from the idf database that have 
                     idf lower than f, but which are not stopwords.
                     The idf I used to sanity check AQUAINT was 1.5

=head1 DESCRIPTION

Inverse document frequency from a corpus, and related 

=head1 NOTES

=head1 SEE ALSO

=head1 BUGS

=cut
    ;

#> 
#< header

BEGIN {
    use Exporter ();
    our (@EXPORT, @ISA);
    @ISA = qw(Exporter);
    @EXPORT = qw(idf idfs);
    @EXPORT_OK = qw(is_stopword words contentwords lowfirst nonstopwords unique fmeas trim regexof);
}
use strict;
use English;
use Carp;
use Data::Dumper;
use Getopt::Long;
use dbm;
#>

#< %Stopword
our %Stopword = ();
chomp, $Stopword{$_} = 1 while <Idf::DATA>;
close Idf::DATA;
sub is_stopword { my $key = shift; return defined $Stopword{$key}; }
#>

#< &idf, &idfs
# count summaries generated
# by src/corpus-stats/generate-idf-hash.pl (v1.1):
#   461348884 tokens read
#     2074524 unique entries read
#     1033461 total documents
#  222.387826797858 average count
#  122.646315010094 average doc count
use constant IDF_DOCUMENTS => 1033461;
use constant IDF_AVG => 122.646315010094;
my $DEFAULT_IDF = log(IDF_DOCUMENTS/IDF_AVG);


my $UNIGRAM_DB = q(unigram-idfs-AQUAINT.dbm);

#< find UNIGRAM_DB in @INC
my $IDF;
foreach my $dir (@INC) {
    if (-e qq($dir/$UNIGRAM_DB)) {
	$UNIGRAM_DB = qq($dir/$UNIGRAM_DB);
	last;
    }
}
if ($UNIGRAM_DB =~ /\//) {
    if ($dbm::BERKELEY) {
	$IDF = readdb($UNIGRAM_DB);
    } else { 
	our $VAR1;
	require $UNIGRAM_DB;
	$IDF = $VAR1;
    }
} else {
#    warn "cannot find [$UNIGRAM_DB] in \@INC: ".join(", ", @INC).$/;
    my $COUNTS_FILE = "unigram-doc-counts-AQUAINT.txt";
    foreach my $dir (@INC) {
	if (-e qq($dir/$COUNTS_FILE)) {
	    $COUNTS_FILE = qq($dir/$COUNTS_FILE);
	    last;
	}
    }
    if ($COUNTS_FILE =~ /\//) {
#	warn "Found counts file [$COUNTS_FILE].\n";
	open (COUNTS, "<", $COUNTS_FILE)
	    or die "cannot read [$COUNTS_FILE]: $!\n";
	warn "Building idfs in memory. This may take a while...\n";
	while (<COUNTS>) {
	    chomp;
	    my ($word, $count) = split /\t/;
	    $IDF->{$word} = log(IDF_DOCUMENTS/$count);
	}
    }
}
#>

sub idf {
    my $w = shift;
    return (exists $IDF->{$w}) ? $IDF->{$w} : $DEFAULT_IDF;
}
 
# idfs(list of words) = the sum of idf for the list of words
sub idfs {
    my $sum=0;
    map { $sum += idf($_); } @_;
    return $sum;
}
#>

#< @w = words(string)
sub words {
    my $text = shift;
    confess "cannot get words of an undefined string\n" unless defined $text;
    my @words = ();
    foreach my $w (split /\s+/, $text) {
	$w =~ s/(\d{3}),(\d{3})/$1$2/g;
	if ($w =~ /^[\$]\d+$/) { $w =~ s/^.//; }
	if ($w =~ /^\d+\%$/) { chop $w; push @words, $w; $w = "percent"; }
	if ($w eq "%") { $w = "percent"; }
	if ($w eq "\$") { $w = "dollars"; }
	if ($w =~ /^(\d+)([a-z]+)$/) { push @words, $1; $w = "_".$2; }
	if ($w =~ /^\d+\W$/) { chop $w; }

	if ($w =~ /[a-zA-Z][a-zA-Z]+/) {
	    $w =~ s!^[^a-zA-Z0-9]+!!;
	    $w =~ s![^a-zA-Z0-9]+$!!;
	}

	next if $w =~ /^\W*$/;
	if ($w =~ /^([A-Z][a-z][a-zA-Z]+-)+[A-Z][a-z][a-zA-Z]+$/) {
	    push @words, split /-/, $w;
	    next;
	} else {
	    push @words, $w;
	}
    }
    return @words;
}
#>
#< @w = contentwords(string)  lowfirst, nonstopwords

# lowercasing the first letter of arg
sub lowfirst { my $s=shift; return "\l$s"; } #man perlop

# returns an array with all non-stopwords within arg that are not short,
# and don't have numbers in them, and aren't capitalized stopwords either.
sub contentwords { 
    return grep { ((not defined $Stopword{$_}) and 
		   (/[a-z][a-z]/) and 
		   (not /[a-z]\d+[a-z]/) and 
		   (not defined $Stopword{"\l$_"})) } words(shift);
}

# returns an array with all non-stopwords within arg
# (less restrictive than &contentwords)
sub nonstopwords {
    return grep { /\S/ and not defined $Stopword{$_} } words(shift);
}

#>
#< @list = unique(@list);

# eliminates duplicates from input list
sub unique {
    my %seen=();
    return grep { if ($seen{$_}) {0;} else {$seen{$_}=1;} } @_;
}

#>
#< fmeas(precision, recall, beta)
sub fmeas {
    my ($precision, $recall, $beta) = @_;
    return 0 unless $precision and $recall;
    return (($beta**2 + 1) * $precision * $recall) /
	((($beta**2)* $precision) + $recall);
}

#>	
#< trim(@)
sub trim {
    for my $i (0..$#_) {
	$_[$i] =~ s/\s+/ /gs;
	$_[$i] =~ s/^\s//gs;
	$_[$i] =~ s/\s$//gs;
    }
    return @_;
}
#>
#< $re = regexof(@list)
sub regexof {
    my $r = "\\b(".join("|", 
			sort { length $b <=> length $a } 
			map { chop if /\.$/; quotemeta $_; } 
			@_).")\\b";
    return qr($r);
}
#> 

#< user interaction
unless(caller()) {
    my $splitter = \&words;
    GetOptions ("help" => sub {exec qq(perldoc $0);},
		"Version" => sub {print VERSION;exit 0;},
		"documents" => sub {print IDF_DOCUMENTS.$/; exit 0; },
		"nonstopwords" => sub { $splitter = \&nonstopwords; },
		"contentwords" => sub { $splitter = \&contentwords; },
		"sanity=f" => sub {
		    shift; my $min = shift;
		    while (my ($key, $value) = each %$IDF) {
			if (idf($key) < $min and not defined $Stopword{$key}) {
			    print qq($value\t$key\n);
			}
		    }
		},
		) or die USAGE;
    foreach (unique map { &{$splitter}($_) } @ARGV) {
	print qq($_\t).idf($_).$/;
    }
}
#> 

1;

__DATA__
###
####
####-##-##
####EDT
APW########.####
XIE########.####
NYT########.####
A####
Cx13
Cx11
Cx1f
1stLd
2ndLd
QL

A
A.
a
a.
about
after
although
also
always
am
among
amongst
an
and
another
any
anybody
anyhow
anyone
anything
anyway
anyways
anywhere
apart
appear
are
aren't
around
as
aside
ask
asking
at
away
awfully

B
B.
b
b.
be
became
because
become
becomes
becoming
been
before
being
believe
believes
believed
below
beside
besides
between
beyond
both
brief
bt
bt.
but
But
by
By

C
C.
c
c.
c'mon
c's
came
can
can't
cannot
cant
cause
causes
certain
certainly
changes
clearly
co
com
come
comes
concerning
consequently
consider
considering
contain
containing
contains
could
couldn't
course
currently

D
D.
d
d.
datelined
definitely
described
despite
did
didn't
dl
do
does
doesn't
doing
don't
done
down
downwards
during

E
E.
e
e.
e.g.
each
eg
either
else
elsewhere
enough
entirely
especially
et
etc
etc.
even
ever
every
everybody
everyone
everything
everywhere
ex
exactly
example
except

F
F.
f
f.
far
few
For
for
from
further
furthermore

G
G.
g
g.
get
gets
getting
given
gives
go
goes
going
gone
good
got
gotten

H
H.
h
h.
had
hadn't
happen
happened
happens
hardly
has
hasn't
have
haven't
having
He
he
he's
he'd
he'll
hello
help
hence
her
here
here's
hereafter
hereby
herein
hereupon
hers
herself
hi
him
himself
his
hither
hopefully
how
howbeit
however

i
i.
I
I.
i'd
i'll
i'm
i've
I'd
I'll
I'm
I've
ie
i.e.
if
In
in
inasmuch
inc
inc.
indeed
inner
insofar
instead
into
inward
is
isn't
It
it
it'd
it'll
it's
its
itself

j
j.
J
J.
just

k
k.
K
K.
keep
keeps
kept
kl
know
knows
known

l
l.
L
L.
Ld
last
lately
later
latter
latterly
least
less
lest
let
let's
like
liked
likely
little
look
looking
looks
ltd

m
m.
M
M.
mainly
made
make
many
may
maybe
me
mean
meanwhile
merely
might
ml
more
moreover
most
mostly
much
must
my
myself

n
n.
N
N.
namely
nd
near
nearly
necessary
need
needs
neither
never
nevertheless
New
new
News
NEWS
next
no
nobody
non
none
noone
nor
normally
not
nothing
novel
now
nowhere
n't

o
o.
O
O.
obviously
of
off
often
oh
ok
okay
old
on
once
ones
only
onto
or
other
others
otherwise
ought
our
ours
ourselves
out
outside
over
overall
own

p
p.
P
P.
particular
particularly
per
perhaps
placed
please
plus
possible
presumably
probably
provides

q
q.
Q
Q.
que
quite
qv

r
r.
R
R.
rather
rd
re
really
reasonably
regarding
regardless
regards
relatively
reported
respectively
ri
right
rtd

s
s.
S
S.
said
same
saw
say
saying
says
sb
sd
secondly
see
seeing
seem
seemed
seeming
seems
seen
self
selves
sensible
sent
serious
seriously
shall
She
she
she's 
she'd
she'll
should
shouldn't
since
so
some
somebody
somehow
someone
something
sometime
sometimes
somewhat
somewhere
soon
sorry
specified
specify
specifying
still
story
STORY
sub
such
sup
sure

t
t.
T
T.
t's
taf
take
taken
tell
tends
th
than
thank
thanks
thanx
that
that's
that'd
that'll
thats
The
the
their
theirs
them
themselves
then
thence
there
there's
there'd
there'll
there're
there've
thereafter
thereby
therefore
therein
theres
thereupon
these
they
they'd
they'll
they're
they've
thing
think
this
thorough
thoroughly
those
though
through
throughout
thru
thus
to
together
too
took
toward
towards
tried
tries
truly
try
trying

u
u.
U
U.
un
under
unfortunately
unless
unlikely
until
unto
up
upon
us
use
used
useful
uses
using
usually

v
v.
V
V.
v.v.
value
various
very
via
viz
vs
vs.

w
w.
W
W.
want
wants
was
wasn't
way
we
we'd
we'll
we're
we've
welcome
well
went
were
weren't
what
what's
what'll
what'd
what're
what've
whatever
when
whence
whenever
where
where's
where'll
where'd
where're
where've
whereafter
whereas
whereby
wherein
whereupon
wherever
whether
which
while
whither
who
who's
who'll
who'd
who're
who've
whoever
whole
whom
whose
why
why's
why'll
why'd
why're
why've
will
willing
wish
with
within
without
won't
wonder
would
would
wouldn't
Writethru

x
x.
y
y.
X
X.
Xinhua

Y
Y.
yes
yet
you
you'd
you'll
you're
you've
your
yours
yourself
yourselves

z
z.
Z
Z.
zero
