require File.dirname(__FILE__) + "/../test_helper.rb"

module ExecutableTestHelper

  include TestHelper

  GLOBAL_THRESHOLD = /^GLOBAL\t([0-9.e-]+)$/
  # TODO the qidre is repeated (also in Parseable)
  @@qidre = /[a-zA-Z\d\.]+/
  QUESTION_LEVEL_THRESHOLD = /^QUESTION\t(#{@@qidre})\t([0-9.e-]+)$/
  NUGGET_LEVEL_THRESHOLD = /^NUGGET\t(#{@@qidre})\t([0-9]+)\t([0-9.e-]+)$/

  EXPECTED_F_LINE = /^(#{@@qidre})\t([a-zA-Z.0-9]+)\t([0-9.e-]+)\t([0-9.e-]+)\t([0-9.e-]+)\t([0-9.e-]+)\t([0-9.e-]+)\t([0-9.e-]+)$/
  NUGGET_PROBABILITY_LINE = /^#?NUGGET_PROBABILITY\t(#{@@qidre})\t([0-9]+)\t([0-9.e-]+)$/
  NUGGET_PROBS_LIST_LINE = /^#?NUGGET_PROBS_LIST\t(.*)$/
  QRESULT_RESPONSE_LINE = /^#?QRESULT_RESPONSE\t(.*)$/

  def run_executable(filename, data_file = NEW_FORMAT_DATA_FILE)
    executable = File.dirname(__FILE__) + "/" + filename
    output = `cat #{data_file} | #{executable} 2>&1`
    if $?.exitstatus != 0
      puts output
      fail("Executable blew up -- see above.")
    end
    assert_match /^Starting at/, output
    assert_match /^Done at/, output
    assert_match /^Done at/, output.split("\n")[-1]
    output
  end

  def run_executable_quietly(filename, data_file = NEW_FORMAT_DATA_FILE)
    executable = File.dirname(__FILE__) + "/" + filename
    output = `cat #{data_file} | #{executable} --quiet 2>&1`
    if $?.exitstatus != 0
      puts output
      fail("Executable blew up -- see above.")
    end
    assert_no_match /^Starting at/, output
    assert_no_match /^Done at/, output
    output
  end

  def assert_global_threshold_present(string)
    assert_match GLOBAL_THRESHOLD, string
    assert_equal 1, string.scan(/GLOBAL/).length
  end

  def assert_no_global_thresholds(string)
    assert_no_match /GLOBAL/, string
  end

  def extract_global_threshold(string)
    string.match(GLOBAL_THRESHOLD)[1].to_f
  end
    

  def assert_qid_threshold_present(string)
    assert_match QUESTION_LEVEL_THRESHOLD, string
  end

  def count_qid_thresholds(string)
    string.scan(QUESTION_LEVEL_THRESHOLD).length
  end

  def assert_no_qid_thresholds(string)
    assert_no_match /QUESTION/, string
  end

  def extract_qid_thresholds(string)
    string.scan(QUESTION_LEVEL_THRESHOLD).inject(Hash.new) { |hash, qid_thresh| hash[qid_thresh[0]] = qid_thresh[1].to_f; hash }
  end

  def assert_question_considered(string, question)
    if (question.kind_of? Array)
      question.each do |real_question|
        assert_question_considered(string, real_question)
      end
    else
      regex = /^(question-level|global) won at question #{question}(, nugget [0-9+]|).$/
      assert_match regex, string
      assert_equal 1, string.scan(regex).length
    end
  end

  alias assert_questions_considered assert_question_considered


  def assert_nid_threshold_present(string)
    assert_match NUGGET_LEVEL_THRESHOLD, string
  end

  def count_nid_thresholds(string)
    string.scan(NUGGET_LEVEL_THRESHOLD).length
  end

  def assert_no_nid_thresholds(string)
    assert_no_match /NUGGET/, string
  end

  def extract_nid_thresholds(string)
    string.scan(NUGGET_LEVEL_THRESHOLD).inject(Hash.new { |hash, key| hash[key] = Hash.new }) do |hash, nid_thresh| 
      hash[nid_thresh[0]][nid_thresh[1]] = nid_thresh[2].to_f
      hash
    end
  end

  def assert_nugget_considered(string, question, nugget)
    regex = /^(nugget-level|question-level|global) won at question #{question}, nugget #{nugget}.$/
    assert_match regex, string
    assert_equal 1, string.scan(regex).length
  end

  def assert_nuggets_considered(string, qids_nids)
    qids_nids.each do |pair|
      assert_nugget_considered(string, pair[0], pair[1])
    end
  end

  def expected_f_lines(string)
    string.scan(EXPECTED_F_LINE)
  end

  def expected_f_reports(string)
    expected_f_lines(string).map do |pieces|
      pieces[0..1] + pieces[2..-1].map { |piece| piece.to_f }
    end
  end

  def nugget_probability_lines(string)
    string.scan(NUGGET_PROBABILITY_LINE)
  end

  def nugget_probs_list_lines(string)
    string.scan(NUGGET_PROBS_LIST_LINE).map do |pieces|
      pieces[0..1] + [ pieces[2].to_f ]
    end
  end

  def nugget_probs_lists(string)
    nugget_probs_list_lines(string).map do |line|
      line[0].split(", ").inject(Hash.new(0)) do |hash, nug_prob|
        hash[nug_prob.split("=>")[0]] = nug_prob.split("=>")[1].to_f
        hash
      end
    end
  end

  def qresult_response_lines(string)
    string.scan(QRESULT_RESPONSE_LINE)
  end

  def qresult_responses(string)
    qresult_response_lines(string).map do |line|
      line[0].split("\t").map { |num| num.to_f }
    end
  end
end
