require 'test/unit'
require File.dirname(__FILE__) + "/executable_test_helper.rb"

class FullInferenceTest < Test::Unit::TestCase

  include ExecutableTestHelper

  def test_nid_full_inference
    output = run_executable("nid_full_inference.rb --answer_key test_data/answer_key --runtag Foo", TRAINING_AND_JUDGEMENT_DATA)
    assert_close([ ["3001", "Foo", 1.02281133911139,
                    0.0, 7, 0.34093711303713,
                    0.978884956260849, 0.364207833356271],
                   ["q02", "Foo", 0.977320265550203,
                    0.0, 6, 0.977320265550203,
                    0.977320265550203, 0.977320265550203] ],
                 expected_f_reports(output))
    # To know why other nuggets don't get zeros, look in the answer key.
    assert_close([ ["3001", "2", 0.977891870855549],
                   ["3001", "3", 0],
                   ["3001", "4", 0],
                   ["3001", "7", 0.0449194682558397],
                   ["q02", "1", 0.977320265550203]],
                 nugget_probability_lines(output))

  end

  def test_silence
    output = `cat #{TRAINING_AND_JUDGEMENT_DATA} | nid_full_inference.rb --answer_key test_data/answer_key --quiet 2>&1`
    assert_no_match /^Starting at/, output
    assert_no_match /^Done at/, output
    assert_no_match /^eps_/, output
  end

  def test_nid_full_inference_split_input
    output = run_executable("nid_full_inference.rb --answer_key test_data/answer_key --runtag Bar #{LEVEL_TEST_DATA_FILE}", JUDGEMENT_REQUESTS)
    assert_close([ ["3001", "Bar", 1.02281133911139,
                    0.0, 7, 0.34093711303713,
                    0.978884956260849, 0.364207833356271],
                   ["q02", "Bar", 0.977320265550203,
                    0.0, 6, 0.977320265550203,
                    0.977320265550203, 0.977320265550203] ],
                 expected_f_reports(output))
  end

  def test_nid_infer_online
    output = run_executable("nid_infer_online.rb --answer_key test_data/answer_key --runtag Bar #{LEVEL_TEST_DATA_FILE}", JUDGEMENT_AND_QRESULT_REQUESTS)
    assert_equal 3, nugget_probs_list_lines(output).length
    assert_close( [ {"1" => 0.977320265550203 },
                    {"7" => 0.0227551898720987, "2" => 0.97737881191131},
                    {"7" => 0.0226803746144635, "2" => 0.0226804596746994} ],
                  nugget_probs_lists(output))
    assert_equal 2, qresult_response_lines(output).length
    assert_close([ [ 0.977320265550203, 1, 0.0, 0, 0.977320265550203,
                     1, 6, 97.7320265550203, 0.977320265550203,
                     0.977320265550203, 0.977320265550203 ],
                   [ 1.02281133911139, 3, 0.0, 1, 1.02281133911139,
                     4, 7, 102.281133911139, 0.34093711303713,
                     0.978884956260849, 0.364207833356271 ] ],
                 qresult_responses(output))
  end

  def vary_answer_keys(prog)
    Proc.new do |answer_key_string|
      cmd = "#{prog} #{answer_key_string} " + 
        "--runtag Bar #{SMALL_LEVEL_TEST_DATA_FILE}"
      run_executable_quietly(cmd, JUDGEMENT_AND_QRESULT_REQUESTS)
    end
  end

  def test_answer_keys
    other_answer_key_strings = 
      [ "--answer_keys test_data/answer_key",
        "--answer_key test_data/split_answers_a " + 
        "--answer_key test_data/split_answers_b",
        "--answer_keys test_data/split_answers_a,test_data/split_answers_b" ]
    programs = [ vary_answer_keys("nid_infer_online.rb"),
                 vary_answer_keys("nid_full_inference.rb") ]
    programs.each do |prog|
      base_output = prog.call("--answer_key test_data/answer_key")
      other_answer_key_strings.each do |str|
        assert_equal base_output, prog.call(str), "Answer differed with spec #{str}"
      end
    end
  end

end
