#!/usr/bin/ruby

#require 'profile'
require File.dirname(__FILE__) + "/../data_set.rb"
require File.dirname(__FILE__) + "/../parsing/nugget_data.rb"
require File.dirname(__FILE__) + "/../environment.rb"

# Assumes that epsilons are global and thresholds are per-nugget.
# Collects training data either from stdin or a given file argument.
# Collects nugget vitality information from the file(s) specified with
# --answer_key(s).  Collects the judgement requests that represent the
# full set of a system's responses from stdin.  Then infers the
# probability that the system's responses contain each nugget, as well
# as the system's expected f-measure on each question.

Environment.execute do |options|

  training = ARGV[0] if ARGV[0]
  input = STDIN.readlines
  if(training)
    dataset = DataSet.parse_from_file(training, :nugget)
  else
    dataset = DataSet.parse_from_strings(input, :nugget)
  end
  dataset.verbosity = :quiet if options[:quiet]
  judgement_requests = JudgementRequest.parse_from_strings(input)
  nuggets = NuggetData.parse_from_files(options[:answer_keys])

  qids = judgement_requests.map { |req| req.qid }.uniq

  qids.sort.each do |qid|
    STDERR.puts "Starting qid #{qid}..." unless options[:quiet]
    current_requests = judgement_requests.select { |req| req.qid == qid }
    data_points = current_requests.map { |req| req.data_points }.flatten
    nuggets.nuggets(qid).sort { |nid1, nid2| nid1.to_i <=> nid2.to_i }.each do |nid|
      puts "#NUGGET_PROBABILITY\t#{qid}\t#{nid}\t#{dataset.nugget_probability(nid, data_points)}"
    end
    f_measure = dataset.expected_f(current_requests, nuggets.vitalities[qid])
    puts "#{qid}\t#{options[:runtag]}\t#{f_measure.vital_count}\t#{f_measure.okay_count}\t#{f_measure.length}\t#{f_measure.recall}\t#{f_measure.precision}\t#{f_measure.f_measure}"
  end

end
