#require 'rational'
#require 'mathn'

# A polynomial: \sum_i a[i]x^i

class Polynomial

  attr_reader :coefficients

  def initialize(coefficients)
    @coefficients = coefficients
  end

  def index_of_first_nonzero_coeff
    ans = 0
    ans += 1 while @coefficients[ans] == 0
    ans
  end

  def evaluate(x)
     @coefficients[index_of_first_nonzero_coeff..@coefficients.length].reverse.inject(0) { |total, coeff| total * x + coeff } * x ** index_of_first_nonzero_coeff
#    @coefficients.reverse.inject(0) { |total, coeff| total * x + coeff }
  end

  def antiderivative
    index = 0
    Polynomial.new([0] + @coefficients.map { |coeff| index += 1; Float(coeff)/index })
  end

  def self.one_minus_x_binomial(degree)
#    t = Time.now
    ans = Polynomial.new((0..degree).map { |i| (-1)**i * degree.choose(i) })
#    puts "#{(Time.now - t).to_f}: Polynomial.one_minus_x_binomial with degree #{degree}"
    ans
  end

  def times_x_to_the(n)
    Polynomial.new([0] * n + @coefficients)
  end

  def self.integral_x_one_minus_x_powers_from_zero_to_one(n, m)
    #n.factorial * m.factorial / (n + m + 1).factorial
    1.0 / ((m + n).choose(n) * (n + m + 1))
  end

  def self.integral_x_one_minus_x_powers_from_zero_to_one_log10(n, m)
    0 - (m + n).choose_log10(n) -  (n + m + 1).estimated_log10
  end

  def self.add_timings_to_method(method_name)
    module_eval "
      alias #{(method_name + "_old").to_sym} #{method_name.to_sym}\ndef #{method_name}(*args)\nt = Time.now\nans = #{method_name + "_old"}(*args);\nputs (Time.now - t).to_f.to_s + ': #{method_name}' + if args.length > 0: '(' + args.to_s + ') ' else ' ' end + if @coefficients: 'on a polynomial with ' + @coefficients.length.to_s + ' coefficients' else '' end\nans\nend"
  end

#  add_timings_to_method "antiderivative"
#  add_timings_to_method "evaluate"
end

class Integer
  def factorial
    (1..self).inject(1) { |fact, cur| fact * cur }
  end

  def factorial_log10
    (1..self).inject(0) { |log_fact, cur| log_fact + cur.estimated_log10 }
  end

  def choose(i)
    if 2*i > self
      self.choose(self - i)
    else
      ((self-i+1)..self).inject(1) { |prod, cur| prod * cur } / i.factorial
    end
#    self.factorial / (i.factorial * (self - i).factorial)
  end

  def choose_log10(i)
    if 2*i > self
      self.choose_log10(self - i)
    else
      ((self-i+1)..self).inject(0) { |log_sum, cur| log_sum + cur.estimated_log10 } - i.factorial_log10
    end
  end

end
