require 'test/unit'
require File.dirname(__FILE__) + "/polynomial.rb"

class PolynomialTest < Test::Unit::TestCase
  
  def test_binomial
    assert_equal [1, -5, 10, -10, 5, -1], Polynomial.one_minus_x_binomial(5).coefficients
  end

  def test_evaluate
    assert_equal -9**5, Polynomial.one_minus_x_binomial(5).evaluate(10)
    assert_equal 1/2**5, Polynomial.new([0, 0, 0, 0, 0, 1]).evaluate(1/2)
  end

  def test_antiderivative
    assert_equal [0, 1, -5.0/2, 10.0/3, -10.0/4, 1, -1.0/6], Polynomial.one_minus_x_binomial(5).antiderivative.coefficients
  end

  def test_hairy_function
    assert_in_delta Polynomial.one_minus_x_binomial(1).times_x_to_the(1).antiderivative.evaluate(1), 
    Polynomial.integral_x_one_minus_x_powers_from_zero_to_one(1, 1), 1E-07
    assert_in_delta Polynomial.one_minus_x_binomial(10).times_x_to_the(10).antiderivative.evaluate(1), 
    Polynomial.integral_x_one_minus_x_powers_from_zero_to_one(10, 10), 1E-14
  end

  def test_log_mode
    assert_in_delta Math.log10(5.factorial), 5.factorial_log10, 1E-7
    assert_in_delta 1000.factorial.estimated_log10, 1000.factorial_log10, 1E-7
    assert_in_delta Math.log10(50.choose(42)), 50.choose_log10(42), 1E-7
    assert_in_delta 1000.choose(42).estimated_log10, 1000.choose_log10(42), 1E-7
    assert_in_delta Polynomial.one_minus_x_binomial(1).times_x_to_the(1).antiderivative.evaluate(1).estimated_log10, 
    Polynomial.integral_x_one_minus_x_powers_from_zero_to_one_log10(1, 1), 1E-70
    assert_in_delta Polynomial.one_minus_x_binomial(10).times_x_to_the(10).antiderivative.evaluate(1).estimated_log10, 
    Polynomial.integral_x_one_minus_x_powers_from_zero_to_one_log10(10, 10), 1E-7
    # This one gets screwed up by floating-point arithmetic
    # assert_in_delta Polynomial.one_minus_x_binomial(123).times_x_to_the(123).antiderivative.evaluate(1).estimated_log10, 
    # Polynomial.integral_x_one_minus_x_powers_from_zero_to_one_log10(123, 123), 1E-7
  end

end
