require 'test/unit'
require File.dirname(__FILE__) + "/selective_profiler.rb"

class SelectiveProfilerTest < Test::Unit::TestCase

  @@simple_profiler = SelectiveProfiler.new
  @@profiler = SelectiveProfiler.new
  def self.profiler
    @@profiler
  end

  def profilee1
    # I don't do much
  end

  instrument :profilee1, @@simple_profiler

  def profilee2
    # Neither do I
  end

  instrument :profilee2, @@profiler

  class Subclass
    def profilee3
      # Nor I
    end
    instrument :profilee3, SelectiveProfilerTest.profiler

    def profilee4
      # Nor I
    end
    instrument :profilee4, SelectiveProfilerTest.profiler
  end

  def test_simple_profiling
    assert !@@simple_profiler.stats_collected?
    1.upto(1000) do
      profilee1
    end
    assert @@simple_profiler.stats_collected?
    stats = @@simple_profiler.stats
    assert_equal [SelectiveProfilerTest], stats.keys
    assert_equal [:profilee1], stats[SelectiveProfilerTest].keys
    assert_equal 1000, stats[SelectiveProfilerTest][:profilee1].total_calls
    assert /SelectiveProfilerTest/ === @@simple_profiler.report
    assert !(/Subclass/ === @@simple_profiler.report)
  end

  def test_profiling
    assert !@@profiler.stats_collected?
    1.upto(1000) do
      profilee2
    end
    assert @@profiler.stats_collected?
    1.upto(500) do
      Subclass.new.profilee3
      Subclass.new.profilee4
    end
    assert @@profiler.stats_collected?
    stats = @@profiler.stats
    assert_equal 2, stats.keys.length
    assert stats.has_key?(SelectiveProfilerTest)
    assert stats.has_key?(Subclass)

    assert_equal [:profilee2], stats[SelectiveProfilerTest].keys
    assert_equal 2, stats[Subclass].keys.length
    assert stats[Subclass].has_key?(:profilee3)
    assert stats[Subclass].has_key?(:profilee4)

    assert_equal 1000, stats[SelectiveProfilerTest][:profilee2].total_calls
    assert_equal  500, stats[Subclass][:profilee3].total_calls
    assert_equal  500, stats[Subclass][:profilee3].total_calls
    assert /SelectiveProfilerTest/ === @@profiler.report
    assert /Subclass/ === @@profiler.report
  end

  def test_reporting
    victim = SelectiveProfiler.new
    victim.stats[Class][:bar] = MethodStats.new
    victim.stats[Class][:bar].instance_eval do
      @total_time = 50
      @total_calls = 5
    end
    assert_equal("   calls      sec  ms/call  name\n" +
                 "       5     50.0  10000.0  Class#bar", victim.report)
  end

  def test_reporting_large_numbers
    victim = SelectiveProfiler.new
    victim.stats[Class][:bar] = MethodStats.new
    victim.stats[Class][:bar].instance_eval do
      @total_time = 5000
      @total_calls = 2
    end
    assert_equal("   calls      sec  ms/call  name\n" + 
                 "       2   5000.0 2500000.0  Class#bar", victim.report)
  end

end
