class SortedSet

  # Returns the greatest object in the set that is less than or equal
  # to the given key, or nil if there isn't one.
  def next_smaller(key)
    if @hash.respond_to? :bound
      ans = @hash.upper_bound(key)
      if ans then ans[0] else nil end
    else
      (@keys = @hash.keys).sort! unless @keys
      @keys.binary_search(key)
    end
  end

  # Returns the least object in the set that is greater than or equal
  # to the given key, or nil if there isn't one.
  def next_larger(key)
    if @hash.respond_to? :bound
      ans = @hash.lower_bound(key)
      if ans then ans[0] else nil end
    else
      (@keys = @hash.keys).sort! unless @keys
      @keys.binary_search(key, :find => :next_larger)
    end
  end

  # Returns the next smaller and the next larger objects in the set as
  # an ordered pair.  Both comparisons are strict --- ignores objects
  # equal to the given key.  If the key is larger or
  # smaller than all the objects in the set, the corresponding
  # neighbor will be nil.
  def neighbors(key)
    [next_smaller(key), next_larger(key)]
  end

end
