require File.dirname(__FILE__) + "/parseable.rb"
require File.dirname(__FILE__) + "/judgement_request.rb"

# The vitalities and descriptions of a bunch of nuggets.  This is
# primarily a parser class, whose main job is to extract that
# information from TREC answer key files.  The vitalities and the
# descriptions are presented as hashes, each from qid to a hash from
# nid to the vitality or the description, respectively.

class NuggetData

  extend Parseable

  attr_reader :vitalities, :strings

  def initialize
    @vitalities = Hash.new { |hash, key| hash[key] = Hash.new }
    @strings = Hash.new { |hash, key| hash[key] = Hash.new }
  end

  def self.parse_from_strings(strings)
    ans = NuggetData.new
    strings.each do |str|
      ans.parse_line!(str)
    end
    ans.freeze # Do I need this?
  end

  def parse_line!(line)
    if line =~ /^(#{NuggetData.qidre})\s+([0-9]+)\s+(vital|okay)\s+(.*)$/
      qid = $1
      nid = $2
      vitality = $3
      string = $4
      @vitalities[qid][nid] = vitality == "vital"
      @strings[qid][nid] = string
    end
  end

  # TODO add more options, controlling, for instance, the probability
  # distribution on the number of judgements (= system responses) per
  # question, and on the scores of the nuggets.
  def random_judgement_requests(options = {})
    options[:questions] ||= @vitalities.keys
    if options[:questions].length != 1
      options[:questions].map { |qid| random_judgement_requests options.merge(:questions => [qid] ) }.flatten
    else
      qid = options[:questions][0]
      nugget_scores = vitalities[qid].keys.inject(Hash.new) { |hash, nid| hash[nid] = rand; hash }
      [ JudgementRequest.new(qid, nugget_scores, "Random judgement request") ]
    end
  end

  def num_vital(qid)
    @vitalities[qid].values.select { |vitality| vitality }.length
  end

  def num_okay(qid)
    @vitalities[qid].values.select { |vitality| !vitality }.length
  end

  def nuggets(qid)
    @vitalities[qid].keys
  end

  def vital_nuggets(qid)
    nuggets(qid).select { |nid| @vitalities[qid][nid] }
  end

  def okay_nuggets(qid)
    nuggets(qid).select { |nid| !@vitalities[qid][nid] }
  end

end
