require File.dirname(__FILE__) + "/parseable.rb"

# Something that parses line by line using a regular expression.
# Relies on a regex method (that returns the regular expression in
# question) and provides line_matches?, parse_line, and all the
# derived methods in LineParseable and Parseable.  The client can
# define a parse_regex_groups method that creates an appropriate
# object out of the groups in the client's regex.  By default,
# parse_regex_groups just forwards those groups to the new method.
module RegexParseable

  include LineParseable

  def line_matches?(line)
    line =~ regex
  end

  def parse_line(line)
    if line =~ regex
      parse_regex_groups *($~[1..-1])
    end
  end

  def parse_regex_groups(*args)
    new *args
  end

end
