# A single response to a Trec relationship/ciqa question.  Implicitly
# dependent on the context of the run it was in, as it includes
# information on how it was judged in that run.
class Response
  extend RegexParseable

  attr_reader :question_id, :response_id, :string

  def self.regex
    /^([0-9]+)\s+(.+?)\s+([0-9]+)\s+(.+?)\s+(.+)$/
  end
  
  def initialize(qid, runtag, rid, doc_id, string)
    @question_id = qid
    @system_name = runtag
    @response_id = rid
    @doc_id = doc_id
    @string = string.downcase.gsub(/\s+/, " ")
    @nugget_judgements = Hash.new(:no)
    @nugget_scores = Hash.new(0)
  end

  def assign_nugget!(nugget_id)
    @nugget_judgements[nugget_id] = :yes
  end

  def nugget_has_been_assigned!(nugget_id)
    if @nugget_judgements[nugget_id] == :no
      @nugget_judgements[nugget_id] = :maybe
    end
  end

  def judgement(nugget_id)
    @nugget_judgements[nugget_id]
  end

  def nuggets_assigned
    @nugget_judgements.keys.select { |nid| @nugget_judgements[nid] == :yes }.map { |nid| "#{@question_id}-#{nid}"}.sort
  end

  def any_nugget_assigned?
    @nugget_judgements.values.include?(:yes)
  end

  def add_score!(nugget_id, score)
    @nugget_scores[nugget_id] = score
  end

  def print_data_points
    @nugget_scores.each { |key, value| puts @question_id + "\t" + key + "\t" + value + "\t" + @nugget_judgements[key].to_s }
    (@nugget_judgements.keys - @nugget_scores.keys).each { |key| puts @question_id + "\t" + key + "\t0\t" + @nugget_judgements[key].to_s }
  end

  def trec_length
    string.tr(" ", "").length
  end

end
