require File.dirname(__FILE__) + "/counters.rb"
require File.dirname(__FILE__) + "/data_group.rb"

class Threshold

  attr_reader :data_group, :index, :counters, :interval_length

  def initialize(data_group, index, counters)
    @data_group = data_group
    @data_points = @data_group.data_points
    @index = index # Of the minimum data point whose score exceeds this threshold
    @counters = counters
#    @definite = compute_definite?
    @interval_length = compute_interval_length
  end

  def lower_bound
    if @index <= 0
      0
    else
      @data_points[@index - 1].score
    end
  end

  def upper_bound
    if @index == @data_points.length
      1
    else
      @data_points[@index].score
    end
  end

  def cutoff
    case @index
    when 0: @data_points[@index].score / 2.0
    when @data_points.length: (1 + @data_points[@index - 1].score) / 2.0
    else if @data_points[@index].score <= @data_points[@index - 1].score
           @data_points[@index].score
         else
           (@data_points[@index].score + @data_points[@index - 1].score) / 2.0
         end
    end
  end

#  def definite?
#    @definite
#  end

  def definite?
    case @index
    when 0: true
    when @data_points.length: true
    else @data_points[@index].score > @data_points[@index - 1].score
    end
  end

  def last?
    @index == @data_points.length
  end

  def compute_interval_length
    case @index
    when 0: @data_points[@index].score
    when @data_points.length: 1 - @data_points[@index - 1].score
    else if definite?
           @data_points[@index].score - @data_points[@index - 1].score
         else
           0
         end
    end
  end

  def next_threshold
    raise "Trying to next_threshold the last threshold!" if last?
    Threshold.new(@data_group, @index + 1, counters.step_over(@data_points[index].judgement))
  end

  # To ensure that all threshold walks are over the same threshold
  # objects.
  memoize :next_threshold

  def next_definite
    ans = self.next_threshold
    while !ans.definite?
      ans = ans.next_threshold
    end
    ans
  end

  def unnormalized_posterior_log10(eps_low, eps_high = eps_low)
    counters.probability_log10(eps_low, eps_high) + Math.log10(interval_length)
  end

  def yes_probability(score, eps_low, eps_high)
    case
    when score < lower_bound: eps_low
    when score > upper_bound: 1 - eps_high
    else ((score - lower_bound) * (1 - eps_high) + (upper_bound - score) * eps_low) / interval_length
    end
  end

  def posterior_probability(eps_low, eps_high, normalization_constant_log10)
    10.0 ** (unnormalized_posterior_log10(eps_low, eps_high) - normalization_constant_log10)
  end

  def to_s
    "#{cutoff}"
  end

end
