require 'test/unit'
require File.dirname(__FILE__) + "/data_group.rb"

class ThresholdTest < Test::Unit::TestCase

  include TestHelper

  def setup
    @data = DataGroup.parse_from_file(ARTIFICIAL_POINTS)
    @one_data_point = DataGroup.new([ DataPoint.new(0, 0, 0.5, :yes) ])
  end

  def test_cutoffs
    assert_close_arrays [0.05, 0.15, 0.35, 0.75, 1.0], @data.thresholds.map { |th| th.cutoff }
  end

  def test_trivial_posteriors
    norm_const = @one_data_point.posterior_normalization_constant_log10(0.1, 0.1)
    assert_equal 0.5, 10.0**norm_const
    posteriors = @one_data_point.intervaled_thresholds.map { |th| th.posterior_probability(0.1, 0.1, norm_const) }
    assert_in_delta 1, posteriors.inject(0) { |sum, post| sum + post }, 1E-10
    assert_close_arrays [0.9, 0.1], posteriors
  end

  def test_posterior_probabilities
    norm_const = @data.posterior_normalization_constant_log10(0.1, 0.1)
    posteriors = @data.intervaled_thresholds.map { |th| th.posterior_probability(0.1, 0.1, norm_const) }
    assert_in_delta 1, posteriors.inject(0) { |sum, post| sum + post }, 1E-10
    assert_close_arrays [0.00335570469798658, 0.0302013422818792, 0.815436241610738, 0.151006711409396], posteriors
  end

end
