#!/usr/bin/perl -w

use strict;
use Carp;
use Data::Dumper;
use Getopt::Long;
#< documentation
use constant VERSION => q$Id: pl.tmpl,v 1.3 2004/07/16 17:19:32 gremio Exp $.$/;
use constant USAGE => <<"EOT"
usage: $0 nuggeteer-output-files
EOT
    ;
=pod

=head1 NAME

heatmap.pl - generate a heat-map visualization of nuggets found

=head1 SYNOPSYS

heatmap.pl nuggeteer-output-files > heatmap.html

=head1 DESCRIPTION

creates a visualization, where every row is one of the output files,
and every column is a nugget (in larger sections by qid), showing in
colors how many times each nugget was found.

=head1 NOTES

It would be nice to have this read the system-output file that was
getting scored, and the answer key, and display in a tooltip the
actual answers and what they did or did not match.

=head1 SEE ALSO

=head1 BUGS

Expects the color images to be in a colors/ directory near the destination.
Right now the only place this works is gremio's public_html directory. 

=cut
    ;
#> 

my $width = 5;
GetOptions ("help" => sub {exec qq(perldoc $0);},
	    "Version" => sub {print VERSION;exit 0;},
	    "size" => \$width,
	    ) or die USAGE;


#< tag(name, %attrs)
sub tag {
    my ($name, %attrs) = @_;
    my $out = qq(<$name);
    foreach my $key (keys %attrs) {
	$out .= qq( $key="$attrs{$key}");
    }
    $out .= qq(>);
    return $out;
}
#>

my @files = ();
my @names = ();
my @qids = ();
foreach my $nugget_file (@ARGV) {
    open (NUG, "<", $nugget_file) or die "cannot read [$nugget_file]: $!\n";
    push @names, $nugget_file;
    my @questions = ();
    my $lastqid;

    my $nuggets_found = {};
    while (<NUG>) {
	next unless /^\#JUDGEMENT\t/;
	my ($foo, $qid, $rid, $nid, $j, $score)
	    = split /\t/;
	
	if (defined $lastqid and $qid ne $lastqid) {
	    push @questions, $nuggets_found;
	    push @qids, $lastqid;
	    $nuggets_found = {};
	}	    

	$nuggets_found->{$nid}||=0;
	$nuggets_found->{$nid}++ if $j;
	
	$lastqid=$qid;
    }
    # last one
    push @questions, $nuggets_found; 
    push @qids, $lastqid;

    push @files, \@questions;
}

print <<"EOH";
<html>
<head><title>heat map</title></head>
<body bgcolor="#FFFFFF">
EOH
    ;

foreach my $question_set (@files) {
    print qq(<!-- ).shift(@names).qq( -->);
    foreach my $nuggets_found (@$question_set) {
	my $qid = shift @qids;
	print tag("img", src => "colors/white.png");
	foreach my $nid (sort {$a <=> $b} keys %$nuggets_found) {
	    my $color = "grey";
	    $color = "black"  if $nuggets_found->{$nid} > 0;
	    $color = "blue"   if $nuggets_found->{$nid} > 1;
	    $color = "purple" if $nuggets_found->{$nid} > 2;
	    $color = "red" if $nuggets_found->{$nid} > 3;
	    $color = "green" if $nuggets_found->{$nid} > 4;
	    $color = "yellow" if $nuggets_found->{$nid} > 5;
	    print tag("img", 
		      width => $width, 
		      src => qq(colors/$color.png),
		      alt => qq($qid:$nid=$nuggets_found->{$nid}));
	}
    }
    print qq(<br>\n);
}

print <<"EOF";
</body>
EOF
    ;

1;
