# -*- perl -*-

# Before `make install' is performed this script should be runnable
# with `make test'. After `make install' it should work as `perl
# external_judgements.t'

#########################

# change 'tests => 1' to 'tests => last_test_to_print';

use Test::More tests => 13;

#########################

require "t/test_helper.pl";

my $pwd=`pwd`; chomp($pwd);
my $dir = qq($pwd/t/tiny-set);
my $jdgdir = qq($dir/official-tiny-set);
my $ansfile = qq($dir/official-tiny-set.answers);
my $outfile = qq($pwd/t/output/tiny-set-ext-judge);
my $errfile = qq($pwd/t/output/tiny-set-ext-judge.error);

my $n = qq(nuggeteer.pl --outputfile $outfile --quiet);

sub run_command {
    my ($cmd, $message) = @_;
    runok($cmd, $outfile, $errfile, $message);
}

run_mock_script();
check_mock_script_results();
run_mocked_nuggeteer();
check_mocked_nuggeteer_results();
run_nuggeteer_with_crashing_subprogram();
check_nuggeteer_with_crashing_subprogram_results();
if (program_runs("src/bayes-threshold/ruby/bin/nid_infer_online.rb")) {
    run_nuggeteer();
    check_nuggeteer_results();
} else {
    pass("Integration will not work because integree doesn't run.");
    pass("Integration will not work because integree doesn't run.");
}

sub run_mock_script {
    run_command(qq(echo "JUDGE_REQUEST" | t/mock-judging.pl --quiet --answer_keys fake,answer,keys t/golden/tiny-set-training), "mock program");
}

sub run_mocked_nuggeteer {
    run_command(qq($n --exclude --bigrams --threshold 0.1 --external-scorer t/mock-judging.pl $ansfile $jdgdir/* $jdgdir/MITCSAIL03.c), "nuggeteer with mock program");
}

sub run_nuggeteer_with_crashing_subprogram {
    $cmd = qq($n --exclude --bigrams --threshold 0.1 --external-scorer t/mock-crash.pl $ansfile $jdgdir/* $jdgdir/MITCSAIL03.c > $outfile 2> $errfile);
    if(system($cmd)) {
	pass("nuggeteer with crashing sub-program");
    } else {
	fail("unexpected successful exit after crashing subprogram");
    }
}

sub run_nuggeteer {
    unlink("/tmp/tinyset.jdb");
    run_command(qq($n --settings tinyset --threshold arj t/modified-csail03c-input), "nuggeteer with real program");
}

sub check_mock_script_results {
    is(`cat $outfile`, "NUGGET_PROBS_LIST\t1=>1, 2=>1, 3=>1, 4=>1, 5=>1, 6=>1, 7=>1, 8=>1, 9=>1, 10=>1, 11=>1, 12=>1\n");
    is(`cat $errfile`, "STDERR Mockery!\n");
}

sub check_mocked_nuggeteer_results {
    my $real_outfile = $outfile . ".exts:t-mock-judging.pl";
    my $output = `cat $real_outfile`;
    my @outstrings = split("\n", $output);
    is($outstrings[@outstrings-2],
       "1905\tMITCSAIL03c\t0.000000000\t0.000000000\t".
       "0\t0.000000000\t0.000000000\t0.000000000");
    is($outstrings[@outstrings-1],
       "1907\tMITCSAIL03c\t0.000000000\t0.000000000\t".
       "0\t0.000000000\t0.000000000\t0.000000000");
    ok((`cat $errfile` eq "" or
	`cat $errfile` eq "Building idfs in memory. This may take a while...\n"),
       "mocked nuggeteer errors as expected");
}

sub check_nuggeteer_with_crashing_subprogram_results {
    my $error_out = `cat $errfile`;
    ok($error_out =~ /caught sigpipe \(a program that nuggeteer called died\) with message:/, "sigpipe statement present");
    ok($error_out =~ "STDERR Mockery!", "subprogram stderr present");
    ok($error_out =~ "that message also lives in ", "error file pointer present");
}

sub check_nuggeteer_results {
    my $real_outfile = $outfile . ".arj";
    same_output("$pwd/t/golden/tiny-set.arj", $real_outfile, "bayes-judgements");
}

