# -*- perl -*-

# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl external_thresholds.t'

#########################

use Test::More tests => 19;

#########################

require "t/test_helper.pl";

my $pwd=`pwd`; chomp($pwd);
my $dir = qq($pwd/t/tiny-set);
my $jdgdir = qq($dir/official-tiny-set);
my $ansfile = qq($dir/official-tiny-set.answers);
my $outfile = qq($pwd/t/output/tiny-set-ext-thresh);
my $errfile = qq($pwd/t/output/tiny-set-ext-thresh.err);
my $outfile_with_extension = "$outfile.extt:t-mock-thresholds.pl";

my $n = qq(nuggeteer.pl --outputfile $outfile --quiet);

# Ruthlessly copied from external_judgements.t
sub run_command {
    my ($cmd, $message) = @_;
    runok($cmd, $outfile, $errfile, $message);
}

run_mocked_nuggeteer();
check_reported_global_threshold();
check_receipt_of_question_threshold();
check_receipt_of_nugget_thresholds();
if (program_runs("src/bayes-threshold/ruby/bin/simple_gl_qid_nid_selection_thresholds.rb")) {
    run_nuggeteer();
    check_nuggeteer_results();
} else {
    pass("Integration will not work because integree doesn't run.");
    pass("Integration will not work because integree doesn't run.");
}

sub run_mocked_nuggeteer {
    $cmd = qq($n --exclude --bigrams --external-thresholder t/mock-thresholds.pl $ansfile $jdgdir/* $jdgdir/MITCSAIL03.c);
    run_command($cmd, "nuggeteer and mock program");
}

sub check_reported_global_threshold {
    @global_lines = `grep GLOBAL $outfile_with_extension`;
    is(@global_lines, 1, "Only one global threshold reported.");
    if($global_lines[0] =~ /^\#EXTERN: GLOBAL\t(.*)$/) {
	pass("Global threshold reported in the right format.");
	is($1, 0.15, "Global threshold was correctly received.");
    } else {
	fail("Global threshold reported in the wrong format.");
    }
}

sub check_receipt_of_question_threshold {
    @question_lines = `grep "EXTERN: QUESTION" $outfile_with_extension`;
    is(@question_lines, 1, "Only one question threshold reported.");
    if($question_lines[0] =~ /^\#EXTERN: QUESTION\t(.*)\t(.*)$/) {
	pass("Question threshold reported in the right format.");
	is($1, 1905, "Question threshold was correctly received.");
	is($2, 0.05, "Question threshold was correctly received.");
    } else {
	fail("Question threshold reported in the wrong format.");
    }
}

sub check_receipt_of_nugget_thresholds {
    @nugget_lines = `grep NUGGET $outfile_with_extension`;
    is(@nugget_lines, 2, "Only two nugget thresholds reported.");
    if($nugget_lines[0] =~ /^\#EXTERN: NUGGET\t(.*)\t(.*)\t(.*)$/) {
	pass("Nugget threshold reported in the right format.");
	is($1, 1905, "Nugget threshold was correctly received.");
	is($2, 2, "Nugget threshold was correctly received.");
	is($3, 0.0001, "Nugget threshold was correctly received.");
    } else {
	fail("Nugget threshold reported in the wrong format.");
    }
    if($nugget_lines[1] =~ /^\#EXTERN: NUGGET\t(.*)\t(.*)\t(.*)$/) {
	pass("Nugget threshold reported in the right format.");
	is($1, 1907, "Nugget threshold was correctly received.");
	is($2, 3, "Nugget threshold was correctly received.");
	is($3, 0.1, "Nugget threshold was correctly received.");
    } else {
	fail("Nugget threshold reported in the wrong format.");
    }
}

sub run_nuggeteer {
    unlink("/tmp/tinyset.jdb");
    $cmd = qq($n --settings tinyset --threshold art t/modified-csail03c-input);
    run_command($cmd, "nuggeteer with real program");
}

sub check_nuggeteer_results {
    my $real_outfile = $outfile . ".art";
    same_output("$pwd/t/golden/tiny-set.art", $real_outfile, "bayes-thresholds");
}
